% konstante, SI jedinice osim energije (eV)
hbar=1.055e-34;
m=9.11e-31;
q=1.6e-19;

% 1D mreza
L=10e-9; a=0.1e-10;
N=round(L/a)+1
X=a*[0:1:N-1];

% interakcijski parametar t, defekt U0 i hamiltonijan H
t=(hbar^2)/(2*m*(a^2))/q;
U0=50;
H=(2*t*diag(ones(1,N)))-(t*diag(ones(1,N-1),1))-(t*diag(ones(1,N-1),-1));
U=zeros(N,N); U(round(N/2),round(N/2))=U0;
H=H+U;

% racunanje svojstvenih vrijednosti i vektora, energija i valnih funkcija
[V,D]=eig(H); D=diag(D); [Enum,ind]=sort(D);

% racunanje za tri odabrana slucaja
E1=D(ind(1)); psi1=abs(V(:,ind(1))); P1=psi1.*conj(psi1)./a;
E2=D(ind(2)); psi2=abs(V(:,ind(2))); P2=psi2.*conj(psi2)./a;
E3=D(ind(3)); psi3=abs(V(:,ind(3))); P3=psi3.*conj(psi3)./a;

% energijske razine, analiticki model
Ean=(((hbar*pi)^2)/(2*m*(L^2))/q)*[1:N].*[1:N];

figure(1),...
subplot(3,2,1), plot(X/1e-9,P1/1e8,'b-','LineWidth',2), xlim([0 10]),...
    legend('n = 1'),xlabel('x (nm)'),ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'),...
    grid on, title(['Defekt na x = 5 nm, U_{0} = ',num2str(U0),' eV']);
subplot(3,2,3), plot(X/1e-9,P2/1e8,'r--','LineWidth',2), xlim([0 10]),...
    legend('n = 2'),xlabel('x (nm)'),ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'),...
    grid on,...
subplot(3,2,5), plot(X/1e-9,P3/1e8,'k-.','LineWidth',2), xlim([0 10]),...
    legend('n = 3'),xlabel('x (nm)'),ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'),...
    grid on,...
subplot(3,2,[2,4,6]), plot(ind,Enum,'bx',ind,Ean,'r','LineWidth',2),...
    ylim([0 1]), xlabel('n'), ylabel('E (eV)'), grid on,...
    legend('num. s defektom','analit. bez defekta','Location','northwest');
   