% konstante, SI jedinice osim energije (eV)
hbar=1.055e-34;
m=0.2*9.110e-31;
q=1.6e-19;

% 1D mreza
L1=3e-9; L2=4e-9; L3=3e-9;
a=0.1e-10;
N1=round(L1/a); N2=round(L2/a); N3=round(L3/a);
N=N1+N2+N3;
X=a*[0:1:N-1];

% interakcijski parametar t, potencijalni profil UB i hamiltonijan H
t=(hbar^2)/(2*m*(a^2))/q;
UB=1;
H=(2*t*diag(ones(1,N)))-(t*diag(ones(1,N-1),1))-(t*diag(ones(1,N-1),-1));
U=diag([UB*ones(1,N1), zeros(1,N2), UB*ones(1,N3)]);
H=H+U;

% racunanje svojstvenih vrijednosti i vektora, energija i valnih funkcija
[V,D]=eig(H); D=diag(D); [Enum,ind]=sort(D);

% racunanje za tri odabrana slucaja
E1=D(ind(1)); psi1=abs(V(:,ind(1))); P1=psi1.*conj(psi1)./a;
E2=D(ind(2)); psi2=abs(V(:,ind(2))); P2=psi2.*conj(psi2)./a;
E3=D(ind(3)); psi3=abs(V(:,ind(3))); P3=psi3.*conj(psi3)./a;

% integrali kvadriranih valnih funkcija u barijerama
I1=(sum(P1(1:300))+sum(P1(700:1000)))*a;
I2=(sum(P2(1:300))+sum(P2(700:1000)))*a;
I3=(sum(P3(1:300))+sum(P3(700:1000)))*a;

% energijske razine, analiticki model
Ean=(((hbar*pi)^2)/(2*m*(L2^2))/q)*[1:N].*[1:N];
Ean2=(((hbar*pi)^2)/(2*m*((L1+L2+L3)^2))/q)*[1:N].*[1:N];

figure(1),...
subplot(3,2,1), plot(X/1e-9,P1/1e8,'b-','LineWidth',2),...
    xlim([0 max(X/1e-9)]), ylim([0 5]), legend('n = 1'), xlabel('x (nm)'),...
    ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'), grid on,...
subplot(3,2,3), plot(X/1e-9,P2/1e8,'r--','LineWidth',2),...
    xlim([0 max(X/1e-9)]), ylim([0 5]), legend('n = 2'), xlabel('x (nm)'),...
    ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'), grid on,...
subplot(3,2,5), plot(X/1e-9,P3/1e8,'k-.','LineWidth',2),...
    xlim([0 max(X/1e-9)]), ylim([0 5]), legend('n = 3'), xlabel('x (nm)'),...
    ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'), grid on,...
subplot(3,2,[2,4,6]),...
    plot(ind,Enum,'bx',ind,Ean,'r',ind,Ean2,'g','LineWidth',2),...
    ylim([0 2]), xlim([0 20]), legend('numericko','analit., T = 4 nm',...
    'analit., T = 10 nm','Location','southeast'), xlabel('n'),...
    ylabel('E (eV)'), grid on;
