% konstante, SI jedinice osim energije (eV)
hbar=1.055e-34;
m1=0.3*9.11e-31;
m2=0.8*9.11e-31;
q=1.6e-19;

% 1D mreza
L1=2e-9; L2=5e-9; L3=5e-9;
a=0.1e-10;
N1=round(L1/a); N2=round(L2/a); N3=round(L3/a);
N=N1+N2+N3+2; % dvije dodatne tocke za dva sucelja
X=a*[0:1:N-1];

% interakcijski parametar t, kineticki dio hamiltonijana T
t1=(hbar^2)/(2*m1*(a^2))/q;
t2=(hbar^2)/(2*m2*(a^2))/q;
dglavna = [2*t1*ones(1,N1) (t1+t2) 2*t2*ones(1,N2) (t1+t2) 2*t1*ones(1,N3)];
dgore = [t1*ones(1,N1) t2*ones(1,N2+1) t1*ones(1,N3)];
ddolje = [t1*ones(1,N1) t2*ones(1,N2+1) t1*ones(1,N3)];
T=diag(dglavna)-diag(dgore,1)-diag(ddolje,-1);

% profil potencijala
UB=3; dV=0.5;
U=diag([UB*ones(1,N1) UB/2 linspace(0,dV,N2) (dV+UB)/2 (dV+UB)*ones(1,N3)]);

% ukupni hamiltonijan
T=T+U;

% racunanje svojstvenih vrijednosti i vektora, energija i valnih funkcija
[V,D]=eig(T); D=diag(D); [Enum,ind]=sort(D);

% racunanje za tri odabrana slucaja
E1=D(ind(1)); psi1=abs(V(:,ind(1))); P1=psi1.*conj(psi1)./a;
E2=D(ind(2)); psi2=abs(V(:,ind(2))); P2=psi2.*conj(psi2)./a;
E3=D(ind(3)); psi3=abs(V(:,ind(3))); P3=psi3.*conj(psi3)./a;

% energijske razine, analiticki model
Ean=(((hbar*pi)^2)/(2*m2*(L2^2))/q)*[1:N].*[1:N];
Ean2=(((hbar*pi)^2)/(2*m2*((L1+L2+L3)^2))/q)*[1:N].*[1:N];

figure(1),...
subplot(3,2,1), plot(X/1e-9,P1/1e8,'b-',X/1e-9,diag(U),'m:','LineWidth',2),...
    xlim([0 max(X/1e-9)]), ylim([0 8]), legend('n = 1'), xlabel('x (nm)'),...
    ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'), grid on,...
subplot(3,2,3), plot(X/1e-9,P2/1e8,'r-',X/1e-9,diag(U),'m:','LineWidth',2),...
    xlim([0 max(X/1e-9)]), ylim([0 8]), legend('n = 2'), xlabel('x (nm)'),...
    ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'), grid on,...
subplot(3,2,5), plot(X/1e-9,P3/1e8,'k-',X/1e-9,diag(U),'m:','LineWidth',2),...
    xlim([0 max(X/1e-9)]), ylim([0 8]), legend('n = 3'), xlabel('x (nm)'),...
    ylabel('|\Psi(x)|^{2} (10^{8} 1/m)'), grid on,...
subplot(3,2,[2,4,6]), plot(ind,Enum,'bx',ind,Ean,'r',ind,Ean2,'g',...
    'LineWidth',2), ylim([0 4]), legend('numericko','analit., TSi = 5 nm',...
    'analit., TSi = 12 nm','Location','southeast'), xlabel('n'),...
    ylabel('E (eV)'), title(['dV = ',num2str(dV),' eV']), grid on;
