% parametri, SI jedinice osim energije (eV)
t1=1;
Gamma=1; g=-1i*Gamma/2;
zplus=1i*1e-12;
E=linspace(-8,8,2000);
Np=4; % dimenzija hamiltonijana

% matrice hamiltonijana
H1=-t1*diag(ones(1,Np-1),1)-t1*diag(ones(1,Np-1),-1);
H1(1,Np)=-t1; H1(Np,1)=-t1;

for n=1:length(E)
    
    EE=E(n);
    
    % idealni kontakti, polubeskonacni lanac
    ck=-(EE+zplus)/(2*t1); ka=acos(ck); g=-t1*exp(1i*ka);
    % dvije konfiguracije kontakata
    sigL1=zeros(Np,Np); sigL1(1,1)=g;
    sigR1=zeros(Np,Np); sigR1(3,3)=g;
    sigR2=zeros(Np,Np); sigR2(2,2)=g;
    gamL1=1i*(sigL1-sigL1');
    gamR1=1i*(sigR1-sigR1');
    gamR2=1i*(sigR2-sigR2');

    % Greenove funkcije
    G1=inv(((EE+zplus)*eye(Np))-H1-sigL1-sigR1);
    G2=inv(((EE+zplus)*eye(Np))-H1-sigL1-sigR2);
    
    % spektralne funkcije i DOS po orbitalama
    A1=1i*(G1-G1'); A2=1i*(G2-G2');
    
    D11(n)=A1(1,1)/2/pi; D12(n)=A1(2,2)/2/pi;
    D13(n)=A1(3,3)/2/pi; D14(n)=A1(4,4)/2/pi;
    
    D21(n)=A2(1,1)/2/pi; D22(n)=A2(2,2)/2/pi;
    D23(n)=A2(3,3)/2/pi; D24(n)=A2(4,4)/2/pi;
    
    % transmisija
    TM1(n)=real(trace(gamL1*G1*gamR1*G1'));
    TM2(n)=real(trace(gamL1*G2*gamR2*G2'));
    
end

figure(1), ...
    subplot(1,3,2), ...
        plot(D11,E,'b-',D12,E,'r-',D13,E,'g--',D14,E,'k--',(D11+D12+D13+D14),...
        E,'k-','LineWidth',2), grid on, ylim([-4 4]), xlim([0 1.3]),...
        ylabel('E (eV)'), xlabel('DOS(E) (eV^{-1})'), legend(['D1'],['D2'],...
        ['D3'],['D4'],['ukupno'],'Location','northeast'),...
        title('Prva konfiguracija'),...
    subplot(1,3,3), ...
        plot(D21,E,'b-',D22,E,'r--',D23,E,'g-',D24,E,'k--',(D21+D22+D23+D24),...
        E,'k-','LineWidth',2), grid on, ylim([-4 4]), xlim([0 1.3]),...
        ylabel('E (eV)'), xlabel('DOS(E) (eV^{-1})'), legend(['D1'],['D2'],...
        ['D3'],['D4'],['ukupno'],'Location','northeast'),...
        title('Druga konfiguracija'),...
    subplot(1,3,1), ...
        plot(TM1,E,'b-',TM2,E,'r--','LineWidth',2), grid on, ylim([-4 4]),...
        xlim([0 1.3]), ylabel('E (eV)'), xlabel('T(E)'),...
        legend(['prva konfig.'],['druga konfig.'],'Location','northeast');
    


% konstante i parametri za proracun struje
kBT=0.025;
h=4.135e-15; %eV-s
q=1.6e-19;
dE=E(2)-E(1);

Ef=0;
NV=20; V=linspace(0,3,NV);

I1=zeros(1,NV);
I2=zeros(1,NV);
I3=zeros(1,NV);

for n=1:NV
    Vx=V(n);
    EfL=Ef+Vx/2;
    EfR=Ef-Vx/2;
    fL=1./(1+exp((E-EfL)/kBT));
    fR=1./(1+exp((E-EfR)/kBT));

            for k=1:length(E)

                ck=-(E(k)+zplus-Vx/2)/(2*t1); ka=acos(ck); gL=-t1*exp(1i*ka);
                ck=-(E(k)+zplus+Vx/2)/(2*t1); ka=acos(ck); gR=-t1*exp(1i*ka);
                
                sigL1=zeros(Np,Np); sigL1(1,1)=gL;
                sigR1=zeros(Np,Np); sigR1(3,3)=gR;
                sigR2=zeros(Np,Np); sigR2(2,2)=gR;
                
                gamL1=1i*(sigL1-sigL1');
                gamR1=1i*(sigR1-sigR1');
                gamR2=1i*(sigR2-sigR2');

                G1=inv(((E(k)+zplus)*eye(Np))-H1-sigL1-sigR1);
                G2=inv(((E(k)+zplus)*eye(Np))-H1-sigL1-sigR2);

                TM1(k)=real(trace(gamL1*G1*gamR1*G1'));
                TM2(k)=real(trace(gamL1*G2*gamR2*G2'));
    
            end
    
    I=sum(TM1.*(fL-fR))*dE * (2*q/h); I1(n)=abs(I);
    I=sum(TM2.*(fL-fR))*dE * (2*q/h); I2(n)=abs(I);
    I=sum(1*(fL-fR))*dE * (2*q/h); I3(n)=abs(I);
end

figure(2), ...
        plot(V,I1,'b-','LineWidth',2), grid on, xlim([0 3]), xlabel('U (V)'),...
          ylabel('I (A)'), hold on,...
        plot(V,I2,'r--','LineWidth',2), plot(V,I3,'k:','LineWidth',2),...
          legend('prva konfiguracija','druga konfiguracija','T(E)=1, jedan mod',...
          'Location','northwest');




