% parametri, SI jedinice osim energije (eV)
hbar=1.06e-34;
q=1.6e-19;
m=0.25*9.1e-31;
IE=(q*q)/(2*pi*hbar); % strujna konstanta, 2 za spin NIJE ukljuceno
Ef=0.1;
kT=0.025;
UB0=0.4; % visina barijere
D=0.01; % jakost rasprsenja

% definicija strukture
a=0.2e-9; 
LE=10e-9; LC=10e-9; LB=1e-9; LQW=3e-9;
Ttotal=LE+LC+LB+LQW;
NE=floor(LE/a); NC=floor(LC/a); NB=floor(LB/a); NQW=floor(LQW/a);
Np=NE+NB+NQW+NB+NC;
XX=a*1e9*[0:1:Np-1];

% parametar interakcije za EMA hamiltonijan
t0=(hbar^2)/(2*m*(a^2)*q);

% konstrukcija EMA hamiltonijana
UB=[zeros(NE,1);UB0*ones(NB,1);zeros(NQW,1);UB0*ones(NB,1);zeros(NC,1)];
T=(2*t0*diag(ones(1,Np)))-(t0*diag(ones(1,Np-1),1))-(t0*diag(ones(1,Np-1),-1));
T=T+diag(UB);

% definicija energijskog spektra
NEE=201;
E=linspace(-0.5,0.8,NEE);
dE=E(2)-E(1);
zplus=1i*1e-6;

% predalokacija radi brzeg racunanja
IE1=zeros(1,length(E)); IE1c=zeros(1,length(E));
LDOSc=zeros(length(E),Np); LDOSn=zeros(length(E),Np);

% naponske tocke za racunanje struje i plotanje gustoca stanja i struje
NV=1;
%VV=linspace(0,0.4,NV);
VV=0.10;

for iV=1:NV

    V=VV(iV); mu1=Ef; mu2=Ef-V;
    % profil potencijala od narinutog napona
    U1=V*[0*ones(1,NE) linspace(0,-1,(NB+NQW+NB)) -1*ones(1,NC)];
    U1=U1';

    % Fermijeve funkcije za EMA slucaj, 2 za spin ukljuceno, 1/m2
    f1=(2*m*kT*q/(2*pi*hbar^2)).*log(1+exp((mu1-E)./kT)); 
    f2=(2*m*kT*q/(2*pi*hbar^2)).*log(1+exp((mu2-E)./kT));
    
    for k=1:NEE

        sig1=zeros(Np); sig2=zeros(Np);

        ck=1-((E(k)+zplus-U1(1)-UB(1))/(2*t0)); ka=acos(ck);
        sig1(1,1)=-t0*exp(1i*ka); gam1=1i*(sig1-sig1');

        ck=1-((E(k)+zplus-U1(Np)-UB(Np))/(2*t0)); ka=acos(ck);
        sig2(Np,Np)=-t0*exp(1i*ka); gam2=1i*(sig2-sig2');

        sigin1=f1(k)*gam1; sigin2=f2(k)*gam2;

        G=inv(((E(k)+zplus)*eye(Np))-T-diag(U1)-sig1-sig2); % 1/eV
        Gn=G*(sigin1+sigin2)*G';
        A=1i*(G-G');
        
        % koherentni slucaj
        IE1c(k)=real(trace(gam1*Gn)-trace(sigin1*A)); % spin ukljucen u f1, f2
        DDc=diag(1i*(G-G'))/(2*pi); 
        LDOSc(k,1:Np)=2*(real(DDc))'/a; % ldos, 2 za spin, normalizirano, 1/eV-m

        % samokonzistentna petlja za nekoherentni slucaj
        error=1;
        while(error>1e-4)

            sigS=diag(D*diag(G));
            Gnova=inv(((E(k)+zplus)*eye(Np))-T-diag(U1)-sig1-sig2-sigS);
            err1=sum(sum(abs(G-Gnova)))/(sum(sum(abs(G)+abs(Gnova))));
            G=(0.5*G)+(0.5*Gnova);

            siginS=diag(D*diag(Gn));
            Gnnova=G*(sigin1+sigin2+siginS)*G';
            err2=sum(sum(abs(Gn-Gnnova)))/(sum(sum(abs(Gn)+abs(Gnnova))));
            Gn=(0.5*Gn)+(0.5*Gnnova);

            error=max([err1,err2]);

        end

        G=Gnova; A=1i*(G-G'); Gn=Gnnova;
        IE1n(k)=real(trace(gam1*Gn)-trace(sigin1*A)); % spin ukljucen u f1, f2
        DDn=diag(1i*(G-G'))/(2*pi); 
        LDOSn(k,1:Np)=2*(real(DDn))'/a; % ldos, 2 za spin, normalizirano, 1/eV-m

    end
        
    IIDOSc=IE*abs(IE1c(1,:))'*ones(1,Np);
    IIDOSn=IE*abs(IE1n(1,:))'*ones(1,Np);

    Ic=IE*sum(IE1c)*dE; IIc(iV)=abs(Ic);
    In=IE*sum(IE1n)*dE; IIn(iV)=abs(In);

end

figure(1), plot(VV,IIc,'b',VV,IIn,'r--','LineWidth',2),...
    legend('koherentni','nekoherentni'), axis([0 0.5 0 max([IIc,IIn])]),...
    xlabel('U (V)'), ylabel('I (A/m^{2})'), grid on;

figure(2),...
    subplot(1,2,1), pcolor(XX,E,LDOSc), colorbar('Location','EastOutside'),...
        colormap jet, axis([0 Ttotal/1e-9 -0.5 0.8]),...
        xlabel('x (nm)'), ylabel('E (eV)'), shading interp, hold on,...
        title('koherentni'), plot(XX,U1+UB,'w--','LineWidth',2),...
    subplot(1,2,2), pcolor(XX,E,LDOSn), colorbar('Location','EastOutside'),...
        colormap jet, axis([0 Ttotal/1e-9 -0.5 0.8]),...
        xlabel('x (nm)'), ylabel('E (eV)'), shading interp, hold on,...
        title('nekoherentni'), plot(XX,U1+UB,'w--','LineWidth',2);

figure(3),...
    subplot(1,2,1), pcolor(XX,E,IIDOSc), colorbar('Location','EastOutside'),...
        colormap jet, axis([0 Ttotal/1e-9 -0.5 0.8]),...
        xlabel('x (nm)'), ylabel('E (eV)'), shading interp, hold on,...
        title('koherentni'), plot(XX,U1+UB,'w--','LineWidth',2), hold off,...
    subplot(1,2,2), pcolor(XX,E,IIDOSn), colorbar('Location','EastOutside'),...
        colormap jet, axis([0 Ttotal/1e-9 -0.5 0.8]),...
        xlabel('x (nm)'), ylabel('E (eV)'), shading interp, hold on,...
        title('nekoherentni'), plot(XX,U1+UB,'w--','LineWidth',2), hold off;



