% parametri, SI jedinice osim energije (eV)
hbar=1.06e-34; % Js
q=1.6e-19;
Ef=0.15;
kT=0.025;
D=0.001; % jakost rasprsenja
Eop=0.060; % energija fonona

% definicija strukture
a=0.2e-9;
LS=15e-9; LCH=10e-9; LD=15e-9;
NS=floor(LS/a); NCH=floor(LCH/a); ND=floor(LD/a);
Np=NS+NCH+ND+2;
XX=a*1e9*[0:1:Np-1];
Ttotal=(Np-2)*a;

% parametri interakcije za EMA hamiltonijan
t1=0.2/4; % za hladni uvod
t2=4/4; % za ostatak strukture

% konstrukcija EMA hamiltonijana
UB=[zeros(NS,1); 0.25/2; 0.25*ones(NCH,1); 0.25/2; zeros(ND,1)];
dglavna=[2*t1*ones(1,NS) (t1+t2) 2*t2*ones(1,NCH) (t2+t2) 2*t2*ones(1,ND)];
dgore=[t1*ones(1,NS) t2*ones(1,NCH+1) t2*ones(1,ND)];
ddolje=[t1*ones(1,NS) t2*ones(1,NCH+1) t2*ones(1,ND)];
T=diag(dglavna)-diag(dgore,1)-diag(ddolje,-1);

% kvazi-Fermijeve energije i potencijalni profil od narinutog napona
V=0.1; mu1=Ef; mu2=Ef-V;
U1=V*[zeros(1,NS) 0 linspace(0,-1,NCH) -1 -1*ones(1,ND)]';

% definicija energijskog spektra
NE=501; E=linspace(-0.25,0.4,NE);
zplus=1i*1e-12;
dE=E(2)-E(1);
% Fermijeva funkcija f_0 radi jednostavnosti
f1=1./(1+exp((E-mu1)./kT));
f2=1./(1+exp((E-mu2)./kT));

% predalokacija radi brzeg racunanja i inicijalizacija
Gn=zeros(Np,NE); Gp=zeros(Np,NE);
IIDOS=zeros(length(E),Np); Gn=zeros(Np,Np,length(E));
siginS=0*ones(Np,NE); sigoutS=0*ones(Np,NE);
error=1;

while error>1e-6 % barem 1e-6 za konvergenciju i ispravne rezultate
    
    for k=1:NE

        sig1=zeros(Np); sig2=zeros(Np);

        ck=1-((E(k)+zplus-U1(1)-UB(1))/(2*t1)); ka=acos(ck);
        sig1(1,1)=-t1*exp(1i*ka); gam1=1i*(sig1-sig1');

        ck=1-((E(k)+zplus-U1(Np)-UB(Np))/(2*t2)); ka=acos(ck);
        sig2(Np,Np)=-t2*exp(1i*ka); gam2=1i*(sig2-sig2');

        gamS=diag(siginS(:,k)+sigoutS(:,k));

        G=inv(((E(k)+zplus)*eye(Np))-T-diag(U1+UB)-sig1-sig2+1i*0.5*gamS);

        A=1i*(G-G');
        LDOS(k,1:Np)=2*(real(diag(A)))'/(2*pi)/a; % 2 za spin, norm., 1/eV-m

        Gn(:,k)=real(diag(G*((f1(k)*gam1)+(f2(k)*gam2)+diag(siginS(:,k)))*G'));
        Gp(:,k)=diag(A)-Gn(:,k);

        Gnx(:,:,k)=((G*((f1(k)*gam1)+(f2(k)*gam2)+diag(siginS(:,k)))*G'));
        
    end
    
    % energijski pomak za emisiju i apsorpciju
    pomak=floor(Eop/dE)+1;
    % broj fonona
    Nph=1/(exp(dE*pomak/kT)-1);
    
    % generiranje Sigma_in_S i Sigma_out_S s ukljucenim doprinosima
    % emisije i apsorpcije za elektronsku i šupljinsku korelacijsku
    % funkciju
    Gnemi=Gn(:,[pomak+1:NE]); Gnemi=[Gnemi zeros(Np,pomak)];
    Gnabs=Gn(:,[1:NE-pomak]); Gnabs=[zeros(Np,pomak) Gnabs];
    Gpabs=Gp(:,[pomak+1:NE]); Gpabs=[Gpabs zeros(Np,pomak)];
    Gpemi=Gp(:,[1:NE-pomak]); Gpemi=[zeros(Np,pomak) Gpemi];

    siginSnew = (Nph+1)*D*Gnemi + Nph*D*Gnabs;
    sigoutSnew = (Nph+1)*D*Gpemi + Nph*D*Gpabs;

    error1=max(max(abs(siginSnew-siginS)));
    error2=max(max(abs(sigoutSnew-sigoutS)));
    error=max(error1,error2)

    siginS=siginSnew;
    sigoutS=sigoutSnew;
    
end

% racunanje gustoce struje J(r,E) iz korelacijske funkcije Gn(r,r',E)
% hladni uvod nema isti "t" kao ostatak strukture
for k=1:NE
    for m=1:NS+1
      % J(r,E) u jedinicama A/eV
      IIDOS(k,m)=q*q/(2*pi*hbar)*real(1i*(t1*Gnx(m,m+1,k)-(t1)*Gnx(m+1,m,k)));
    end  
    for m=(NS+2):(Np-1)
      IIDOS(k,m)=q*q/(2*pi*hbar)*real(1i*(t2*Gnx(m,m+1,k)-(t2)*Gnx(m+1,m,k)));
    end

end


figure(1),...
  subplot(1,2,1), pcolor(XX,E,LDOS), colorbar('Location','EastOutside'),...
    colormap jet, axis([0 (Ttotal/1e-9) -0.25 0.4]), xlabel('x (nm)'),...
    ylabel('E (eV)'), shading interp, hold on, title('g(x,E) (1/(eV*m))'),...
    plot(XX,U1+UB,'w--','LineWidth',2),...

  subplot(1,2,2), pcolor(XX,E,IIDOS), colorbar('Location','EastOutside'),...
    colormap jet, axis([0 (Ttotal/1e-9) -0.25 0.4]), xlabel('x (nm)'),...
    ylabel('E (eV)'), shading interp, hold on, title('I(x,E) (A/(eV))'),...
    plot(XX,U1+UB,'w--','LineWidth',2);

